<?php
class Ucd_Component_Activate
{
    public $productName;
    public $activateEmail;
    public $activated;

    protected $_activateEmailOption;

    // Config
    public $apiUrl = 'http://activate.wppluginupdate.com/inquiry/receive/';
    public $verifyInterval = 604800; // weekly
    public $verifyRetryInterval = 86400; // daily,
    public $verifyRetryMaxCount = 3;

    public function __construct(stdClass $options)
    {
        if (Ucd_WpPlugin::DEMO) {
            return;
        }

        // Register weekly cron interval
        add_filter('cron_schedules', array($this, 'addCronSchedule'));

        // Cron actions
        add_action('ucd-cron-verify', array($this, 'verify'));
        add_action('ucd-cron-verifyRetry', array($this, 'verifyRetry'));

        $this->productName = Ucd_WpPlugin::PRODUCT_NAME;

        $this->_activateEmailOption = "ucd_activate_email-{$this->productName}";

        $this->activateEmail = get_option($this->_activateEmailOption, NULL);
        $this->activated = !is_null($this->activateEmail);

        if (!$this->activated) {
            // Abort normal activation
            $options->continue = FALSE;

            add_action('admin_menu', array($this, 'registerMenu'));
        }
    }

    public function registerMenu()
    {
        add_menu_page(Ucd_WpPlugin::TITLE,
            Ucd_WpPlugin::MENU_LABEL,
            Ucd_WpPlugin::CAPABILITY,
            Ucd_WpPlugin::SLUG,
            array($this, 'dispatchActivate'),
            Ucd_WpPlugin::getPluginBaseUrl() . 'images/icon-16.png'
        );

        add_submenu_page(Ucd_WpPlugin::SLUG,
            'Activate',
            'Activate',
            Ucd_WpPlugin::CAPABILITY,
            Ucd_WpPlugin::SLUG,
            array($this, 'dispatchActivate')
        );
    }

    //
    // Activation methods
    //

    public function activate($activateEmail)
    {
        update_option($this->_activateEmailOption, $activateEmail);

        $this->activateEmail = $activateEmail;
        $this->activated = TRUE;
    }

    public function deactivate()
    {
        delete_option($this->_activateEmailOption);
    }

    public function dispatchActivate()
    {
        Ucd_WpPlugin::getInstance()->dispatch('activate');
    }

    //
    // Cron
    //

    public function addCronSchedule($schedules)
    {
        $schedules['ucd-cron-verify'] = array(
            'interval' => $this->verifyInterval,
            'display' => 'Activation check interval',
        );

        $schedules['ucd-cron-verifyRetry'] = array(
            'interval' => $this->verifyRetryInterval,
            'display' => 'Activation check retry interval',
        );

        return $schedules;
    }

    public function verify()
    {
        if (is_null($this->activateEmail)) {
            return;
        }

        try {
            Ucd_WpPlugin::loadApplication();
            Ucd_Application::getService('Activate')->verify(
                $this->activateEmail);
            wp_clear_scheduled_hook('ucd-cron-verifyRetry');
            delete_option('ucd_verify_retry_count');
        } catch (Ucd_Service_Activate_Exception $e) {
            // Schedule retry
            if (!wp_next_scheduled('ucd-cron-verifyRetry')) {
                wp_schedule_event(time(), 'ucd-cron-verifyRetry',
                    'ucd-cron-verifyRetry');
            }
        }
    }

    public function verifyRetry()
    {
        if (!is_null($this->activateEmail)) {
            wp_clear_scheduled_hook('ucd-cron-verifyRetry');
            delete_option('ucd_verify_retry_count');
            return;
        }

        try {
            Ucd_WpPlugin::loadApplication();
            Ucd_Application::getService('Activate')->verify(
                $this->activateEmail);
            wp_clear_scheduled_hook('ucd-cron-verifyRetry');
            delete_option('ucd_verify_retry_count');

        } catch (Ucd_Service_Activate_Exception $e) {
            $retryCount = (int) get_option('ucd_verify_retry_count', 0);
            $retryCount++;

            if ($retryCount < $this->verifyRetryMaxCount) {
               update_option('ucd_verify_retry_count', $retryCount);
            } else {
                wp_clear_scheduled_hook('ucd-cron-verifyRetry');
                delete_option('ucd_verify_retry_count');
                $this->deactivate();
            }
        }
    }
}
